@@ -4,15 +4,16 @@ from sapiopylib.rest.pojo.CustomReport import CustomReportCriteria, AbstractRepo
 from sapiopylib.rest.pojo.datatype.FieldDefinition import FieldType
 from sapiopylib.rest.pojo.eln.ElnExperiment import ElnExperiment, ElnExperimentQueryCriteria
 from sapiopylib.rest.pojo.eln.SapioELNEnums import ElnExperimentStatus, ElnBaseDataType
+from sapiopylib.rest.utils.recordmodel.PyRecordModel import PyRecordModel
 from sapiopylib.rest.utils.recordmodel.RecordModelWrapper import WrappedType
 
+from sapiopycommons.customreport.auto_pagers import CustomReportDictAutoPager
 from sapiopycommons.customreport.custom_report_builder import CustomReportBuilder
 from sapiopycommons.customreport.term_builder import TermBuilder
 from sapiopycommons.datatype.pseudo_data_types import EnbEntryOptionsPseudoDef, NotebookExperimentOptionPseudoDef, \
     NotebookExperimentPseudoDef, ExperimentEntryRecordPseudoDef, EnbEntryPseudoDef
 from sapiopycommons.general.aliases import SapioRecord, UserIdentifier, AliasUtil, FieldValue, \
     ExperimentEntryIdentifier, ExperimentIdentifier
-from sapiopycommons.general.custom_report_util import CustomReportUtil
 from sapiopycommons.general.exceptions import SapioException
 from sapiopycommons.recordmodel.record_handler import RecordHandler
 
@@ -201,7 +202,7 @@ class ExperimentReportUtil:
         criteria = report_builder.build_report_criteria()
 
         ret_val: dict[SapioRecord, int] = {}
-        rows: list[dict[str, FieldValue]] = CustomReportUtil.run_custom_report(context, criteria)
+        rows: list[dict[str, FieldValue]] = CustomReportDictAutoPager(context, criteria).get_all_at_once()
         for row in rows:
             dt: str = row[EnbEntryPseudoDef.DATA_TYPE_NAME__FIELD_NAME.field_name]
             exp_id: int = row[EnbEntryPseudoDef.EXPERIMENT_ID__FIELD_NAME.field_name]
@@ -209,16 +210,20 @@ class ExperimentReportUtil:
                 ret_val[record] = exp_id
         return ret_val
 
+    # CR-47491: Support providing a data type name string to receive PyRecordModels instead of requiring a WrapperType.
     @staticmethod
     def map_experiments_to_records_of_type(context: UserIdentifier, exp_ids: list[ExperimentIdentifier],
-                                           wrapper_type: type[WrappedType]) -> dict[int, list[WrappedType]]:
+                                           wrapper_type: type[WrappedType] | str) \
+            -> dict[int, list[WrappedType] | list[PyRecordModel]]:
         """
         Return a dictionary mapping each experiment id to a list of records of the given type that were used in each
         experiment. If an experiment didn't use any records of the given type then it will be mapped to an empty list.
 
         :param context: The current webhook context or a user object to send requests from.
         :param exp_ids: A list of experiment identifiers.
-        :param wrapper_type: The record model wrapper to use, corresponds to which data type we will query for.
+        :param wrapper_type: The record model wrapper or data type name to use, corresponds to which data type we will
+            query for. If a data type name is provided, the returned records will be PyRecordModels instead of
+            WrappedRecordModels.
         :return: A dictionary mapping each experiment id to a list of records of the given type that were used in that
             experiment.
         """
@@ -227,7 +232,7 @@ class ExperimentReportUtil:
 
         user = AliasUtil.to_sapio_user(context)
         record_handler = RecordHandler(user)
-        data_type_name: str = wrapper_type.get_wrapper_data_type_name()
+        data_type_name: str = AliasUtil.to_data_type_name(wrapper_type)
 
         exp_ids: list[int] = AliasUtil.to_notebook_ids(exp_ids)
         rows = ExperimentReportUtil.__get_record_experiment_relation_rows(user, data_type_name, exp_ids=exp_ids)
@@ -268,7 +273,7 @@ class ExperimentReportUtil:
 
         # Ensure that each experiment appears in the dictionary, even if it has no experiment options.
         options: dict[int, dict[str, str]] = {x: {} for x in exp_ids}
-        results: list[dict[str, FieldValue]] = CustomReportUtil.run_custom_report(context, report)
+        results: list[dict[str, FieldValue]] = CustomReportDictAutoPager(context, report).get_all_at_once()
         for row in results:
             exp_id: int = row[NotebookExperimentOptionPseudoDef.EXPERIMENT_ID__FIELD_NAME.field_name]
             key: str = row[NotebookExperimentOptionPseudoDef.OPTION_KEY__FIELD_NAME.field_name]
@@ -299,7 +304,7 @@ class ExperimentReportUtil:
 
         # Ensure that each entry appears in the dictionary, even if it has no entry options.
         options: dict[int, dict[str, str]] = {x: {} for x in entries}
-        results: list[dict[str, FieldValue]] = CustomReportUtil.run_custom_report(context, report)
+        results: list[dict[str, FieldValue]] = CustomReportDictAutoPager(context, report).get_all_at_once()
         for row in results:
             entry_id: int = row[EnbEntryOptionsPseudoDef.ENTRY_ID__FIELD_NAME.field_name]
             key: str = row[EnbEntryOptionsPseudoDef.ENTRY_OPTION_KEY__FIELD_NAME.field_name]
@@ -332,7 +337,7 @@ class ExperimentReportUtil:
         report = report_builder.build_report_criteria()
 
         ret_val: dict[int, str] = {}
-        results: list[dict[str, FieldValue]] = CustomReportUtil.run_custom_report(context, report)
+        results: list[dict[str, FieldValue]] = CustomReportDictAutoPager(context, report).get_all_at_once()
         for row in results:
             exp_id: int = row[NotebookExperimentPseudoDef.EXPERIMENT_ID__FIELD_NAME.field_name]
             name: str = row["TemplateExperimentName"]
@@ -581,7 +586,7 @@ class ExperimentReportUtil:
         """
         user = AliasUtil.to_sapio_user(context)
         exp_ids: list[int] = []
-        for row in CustomReportUtil.run_custom_report(user, report):
+        for row in CustomReportDictAutoPager(user, report):
             exp_ids.append(row[NotebookExperimentPseudoDef.EXPERIMENT_ID__FIELD_NAME.field_name])
         if not exp_ids:
             return []
@@ -646,4 +651,4 @@ class ExperimentReportUtil:
         report_builder.add_join(records_entry_join, ExperimentEntryRecordPseudoDef.DATA_TYPE_NAME)
         report_builder.add_join(experiment_entry_enb_entry_join, EnbEntryPseudoDef.DATA_TYPE_NAME)
         report_builder.add_join(enb_entry_experiment_join, NotebookExperimentPseudoDef.DATA_TYPE_NAME)
-        return CustomReportUtil.run_custom_report(user, report_builder.build_report_criteria())
+        return CustomReportDictAutoPager(user, report_builder.build_report_criteria()).get_all_at_once()
