@@ -1,50 +1,51 @@
 from collections.abc import Iterable
-from typing import Any
+from typing import Any, TypeAlias
 
 from sapiopylib.rest.User import SapioUser
 from sapiopylib.rest.pojo.DataRecord import DataRecord
-from sapiopylib.rest.pojo.datatype.FieldDefinition import FieldType
+from sapiopylib.rest.pojo.datatype.FieldDefinition import FieldType, AbstractVeloxFieldDefinition
 from sapiopylib.rest.pojo.eln.ElnExperiment import ElnExperiment
 from sapiopylib.rest.pojo.eln.ExperimentEntry import ExperimentEntry
 from sapiopylib.rest.pojo.eln.SapioELNEnums import ElnBaseDataType
+from sapiopylib.rest.pojo.eln.eln_headings import ElnExperimentTab
 from sapiopylib.rest.pojo.webhook.WebhookContext import SapioWebhookContext
 from sapiopylib.rest.utils.Protocols import ElnExperimentProtocol, ElnEntryStep
-from sapiopylib.rest.utils.recordmodel.PyRecordModel import PyRecordModel
+from sapiopylib.rest.utils.recordmodel.PyRecordModel import PyRecordModel, AbstractRecordModel
 from sapiopylib.rest.utils.recordmodel.RecordModelWrapper import WrappedRecordModel, WrappedType, WrapperField
 
 from sapiopycommons.general.exceptions import SapioException
 
-FieldValue = int | float | str | bool | None
+FieldValue: TypeAlias = int | float | str | bool | None
 """Allowable values for fields in the system."""
-RecordModel = PyRecordModel | WrappedRecordModel
+RecordModel: TypeAlias = PyRecordModel | AbstractRecordModel | WrappedRecordModel
 """Different forms that a record model could take."""
-SapioRecord = DataRecord | RecordModel
+SapioRecord: TypeAlias = DataRecord | RecordModel
 """A record could be provided as either a DataRecord, PyRecordModel, or WrappedRecordModel (WrappedType)."""
-RecordIdentifier = SapioRecord | int
+RecordIdentifier: TypeAlias = SapioRecord | int
 """A RecordIdentifier is either a record type or an integer for the record's record ID."""
-DataTypeIdentifier = SapioRecord | type[WrappedType] | str
+DataTypeIdentifier: TypeAlias = SapioRecord | type[WrappedType] | str
 """A DataTypeIdentifier is either a SapioRecord, a record model wrapper type, or a string."""
-FieldIdentifier = WrapperField | str | tuple[str, FieldType]
+FieldIdentifier: TypeAlias = AbstractVeloxFieldDefinition | WrapperField | str | tuple[str, FieldType]
 """A FieldIdentifier is either wrapper field from a record model wrapper, a string, or a tuple of string
 and field type."""
-FieldIdentifierKey = WrapperField | str
+FieldIdentifierKey: TypeAlias = WrapperField | str
 """A FieldIdentifierKey is a FieldIdentifier, except it can't be a tuple, s tuples can't be used as keys in
 dictionaries.."""
-HasFieldWrappers = type[WrappedType] | WrappedRecordModel
+HasFieldWrappers: TypeAlias = type[WrappedType] | WrappedRecordModel
 """An identifier for classes that have wrapper fields."""
-ExperimentIdentifier = ElnExperimentProtocol | ElnExperiment | int
+ExperimentIdentifier: TypeAlias = ElnExperimentProtocol | ElnExperiment | int
 """An ExperimentIdentifier is either an experiment protocol, experiment, or an integer for the experiment's notebook
 ID."""
-ExperimentEntryIdentifier = ElnEntryStep | ExperimentEntry | int
+ExperimentEntryIdentifier: TypeAlias = ElnEntryStep | ExperimentEntry | int
 """An ExperimentEntryIdentifier is either an ELN entry step, experiment entry, or an integer for the entry's ID."""
-FieldMap = dict[str, FieldValue]
+FieldMap: TypeAlias = dict[str, FieldValue]
 """A field map is simply a dict of data field names to values. The purpose of aliasing this is to help distinguish
 any random dict in a webhook from one which is explicitly used for record fields."""
-FieldIdentifierMap = dict[FieldIdentifierKey, FieldValue]
+FieldIdentifierMap: TypeAlias = dict[FieldIdentifierKey, FieldValue]
 """A field identifier map is the same thing as a field map, except the keys can be field identifiers instead
 of just strings. Note that although one of the allowed field identifiers is a tuple, you can't use tuples as
 keys in a dictionary."""
-UserIdentifier = SapioWebhookContext | SapioUser
+UserIdentifier: TypeAlias = SapioWebhookContext | SapioUser
 """An identifier for classes from which a user object can be used for sending requests."""
 
 
@@ -142,23 +143,25 @@ class AliasUtil:
     @staticmethod
     def to_data_field_name(value: FieldIdentifier) -> str:
         """
-        Convert a string or WrapperField to a data field name string.
+        Convert an object that can be used to identify a data field to a data field name string.
 
-        :param value: A string or WrapperField.
+        :param value: An object that can be used to identify a data field.
         :return: A string of the data field name of the input value.
         """
         if isinstance(value, tuple):
             return value[0]
         if isinstance(value, WrapperField):
             return value.field_name
+        if isinstance(value, AbstractVeloxFieldDefinition):
+            return value.data_field_name
         return value
 
     @staticmethod
     def to_data_field_names(values: Iterable[FieldIdentifier]) -> list[str]:
         """
-        Convert an iterable of strings or WrapperFields to a list of data field name strings.
+        Convert an iterable of objects that can be used to identify data fields to a list of data field name strings.
 
-        :param values: An iterable of strings or WrapperFields.
+        :param values: An iterable of objects that can be used to identify a data field.
         :return: A list of strings of the data field names of the input values.
         """
         return [AliasUtil.to_data_field_name(x) for x in values]
@@ -205,20 +208,40 @@ class AliasUtil:
                              f"field with the name \"{field}\",")
 
     @staticmethod
-    def to_field_map_lists(records: Iterable[SapioRecord]) -> list[FieldMap]:
+    def to_field_map(record: SapioRecord, include_record_id: bool = False) -> FieldMap:
         """
-        Convert a list of variables that could either be DataRecords, PyRecordModels,
-        or WrappedRecordModels to a list of their field maps.
+        Convert a given record value to a field map.
 
+        :param record: A record which is a DataRecord, PyRecordModel, or WrappedRecordModel.
+        :param include_record_id: If true, include the record ID of the record in the field map using the RecordId key.
+        :return: The field map for the input record.
+        """
+        if isinstance(record, DataRecord):
+            # noinspection PyTypeChecker
+            fields: FieldMap = record.get_fields()
+        else:
+            # TI-47593: Copy the record's fields by using the get() method instead of copy_to_dict() so that date
+            # macros get translated to valid field values.
+            fields: FieldMap = {f: record.fields.get(f) for f in record.fields}
+        # PR-47457: Only include the record ID if the caller requests it, since including the record ID can break
+        # callbacks in certain circumstances if the record ID is negative.
+        if include_record_id:
+            fields["RecordId"] = AliasUtil.to_record_id(record)
+        return fields
+
+    @staticmethod
+    def to_field_map_list(records: Iterable[SapioRecord], include_record_id: bool = False) -> list[FieldMap]:
+        """
+        Convert a list of variables that could either be DataRecords, PyRecordModels, or WrappedRecordModels
+        to a list of their field maps. This includes the given RecordId of the given records.
+
+        :param records: An iterable of records which are DataRecords, PyRecordModels, or WrappedRecordModels.
+        :param include_record_id: If true, include the record ID of the records in the field map using the RecordId key.
         :return: A list of field maps for the input records.
         """
         field_map_list: list[FieldMap] = []
         for record in records:
-            if isinstance(record, DataRecord):
-                # noinspection PyTypeChecker
-                field_map_list.append(record.get_fields())
-            else:
-                field_map_list.append(record.fields.copy_to_dict())
+            field_map_list.append(AliasUtil.to_field_map(record, include_record_id))
         return field_map_list
 
     @staticmethod
